
/* max31855.c -- Read and show values for the max31855. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdint.h>
#include <math.h>

/* For SPI */
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>

#include "max31855.h"


int readmax31855(char *spidevice, char *info, size_t infomax,
	double *ta, double *tr, uint8_t *fault, uint8_t *ftype) 
{
	char errinfo[128];
	int fd, oks;

	uint8_t mode;
	uint8_t bits;
	uint32_t speed;

	struct spi_ioc_transfer sit[1];
	uint8_t items[5];

	int32_t tcv25, trv625;

	memset(info, 0, infomax); 

	fd = open(spidevice, O_RDWR);
	if(fd < 0)
	{
		strerror_r(errno, errinfo, 128);
		snprintf(info, infomax, "%s: %s\n", spidevice, errinfo); 
		return(-1);	
	}

	/* Max31855 wants mode 1 */
	mode = SPI_MODE_1; 
	oks = ioctl(fd, SPI_IOC_WR_MODE, &mode);

	if(oks < 0)
	{
		strerror_r(errno, errinfo, 128);
		snprintf(info, infomax, "Setting device mode to 0x%x: %s", mode, errinfo); 
		close(fd);
		return(-1);
	}

	bits = 8;
	oks = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
	if(oks < 0)
	{
		strerror_r(errno, errinfo, 128);
		snprintf(info, infomax, "Setting %d bits per word: %s", bits, errinfo); 
		close(fd);
		return(-1);
	}

	speed = 100000;
	oks = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if(oks < 0)
	{
		strerror_r(errno, errinfo, 128);
		snprintf(info, infomax, "Setting speed to %u Hz: %s", speed, errinfo); 
		close(fd);
		return(-1);
	}

	memset(items, 0, 5); 
	memset(&sit, 0, sizeof(sit)); 

	sit[0].tx_buf = (__u64) 0;
	sit[0].rx_buf = (uintptr_t) items;
	sit[0].len = 4;
	sit[0].speed_hz = speed;
	sit[0].bits_per_word = bits;
	sit[0].delay_usecs = 0;
	sit[0].cs_change = 0;

	// fprintf(stderr, "%s(%d): 

	oks = ioctl(fd, SPI_IOC_MESSAGE(1), sit);

	if(oks < 0)
	{
		strerror_r(errno, errinfo, 128);
		snprintf(info, infomax, "read error %s\n", errinfo); 
		close(fd);
		return(-1);	
	}

	close(fd);

	/* Bits: 31-18 2-complement signed temperature value (+1600 to -250)
			 17
             16    1=Fault 0=ok
			 15-4  2-complement signed reference junction temperature (-55 to +127)
			 3
             2     1=Fault: TC shorted to vcc
             1     1=Fault: TC shorted to gnd
             0     1=Fault: TC is open
	*/ 

	tcv25 = (items[0] << 6) | (items[1] >> 2)  ; 
	*fault = (items[1] & 0x1);
	*ftype = (items[3] & 0x07);
	trv625 = (items[2] << 4) | (items[3] >> 4);

	/* Negativ temperature bit  */
	if(items[0] & 0x80 )
	{
		*ta = (double) (tcv25) / 4.0 - 4096.0; 
	}
	else
	{
		*ta = (double) (tcv25) / 4.0; 
	}
	*tr = (double) (trv625) / 16.0; 
		
	/* fprintf(stderr, "%s(%d): T=0x%04x %G C Tref=0x%03x %G C Fault %d type %d\n", 
		__FILE__, __LINE__, tcv25, *ta, trv625, *tr, *fault, *ftype);  */ 

	return(0);
}

/* Tell about failures and status info */
int fdetandinfo31855(int rdok, int fault, int ftype, char *txbuff, size_t tbufmax)
{
	int scv, scg, oc;
	size_t tbx;
	int rvx;

	if(rdok < 0)
	{
		snprintf(txbuff, tbufmax, "Failed to access device"); 
		return(1);
	}

	if(fault == 0)
	{
		snprintf(txbuff, tbufmax, "OK"); 
		return(0);
	}

	scv = ftype & 4;
	scg = ftype & 2;
	oc =  ftype & 1;
	tbx = 0; 
	memset(txbuff, 0, tbufmax); 
	rvx = 0;

	if(scv)
	{
		snprintf(txbuff+tbx, tbufmax-tbx, "Shorted to Vcc "); 
		tbx = strlen(txbuff); 
		rvx ++;
	}

	if(scg)
	{
		snprintf(txbuff+tbx, tbufmax-tbx, "Shorted to GND "); 
		tbx = strlen(txbuff); 
		rvx ++;
	}

	if(oc)
	{
		snprintf(txbuff+tbx, tbufmax-tbx, "Open-circuited "); 
		tbx = strlen(txbuff); 
		rvx ++;
	}

	return(rvx);
}





