
/* spiio.c -- multi-byte SPI for the rfm22. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>
#include <stdint.h>
#include <math.h>

/* For SPI */
#include <sys/ioctl.h>
#include <linux/types.h>
#include <linux/spi/spidev.h>

#include "spiio.h"

/* Put out 16 bits in one go to the device */
int write16(int fd, uint16_t bits16)
{
	int rvv;
	struct spi_ioc_transfer sit[1];
	uint8_t items[2];

	memset(&sit, 0, sizeof(sit));

	items[0] = (bits16 >> 8) & 0xFF;
	items[1] = bits16 & 0xFF;

	sit[0].tx_buf = (uintptr_t) items;
	sit[0].rx_buf = (__u64) 0;
	sit[0].len = 2;
	sit[0].speed_hz = 100000;
	sit[0].bits_per_word = 8;
	sit[0].delay_usecs = 0;
	sit[0].cs_change = 0;

	rvv = ioctl(fd, SPI_IOC_MESSAGE(1), sit);
	return(rvv);
}


/*       Write: RW=1 7-bit address, 8 bit data */
/* Burst write: RW=1 7-bit address, 8 bit*N data */

int writeN(int fd, uint8_t address, uint8_t *bytes, size_t bcount) 
{
	int rvv;
	struct spi_ioc_transfer sit[1];
	size_t j, itemcount;
	uint8_t *items;

	memset(&sit, 0, sizeof(sit));

	itemcount = bcount + 1;
	items = (uint8_t *) calloc(itemcount, sizeof(uint8_t) ); 

	items[0] = (address | 0x80);  /* Bit 7 set for write */ 

	for(j = 0; j<bcount; j++)
	{
		items[j+1] = bytes[j];
	}

	sit[0].tx_buf = (uintptr_t) items;
	sit[0].rx_buf = (__u64) 0;
	sit[0].len = itemcount;
	sit[0].speed_hz = 100000;
	sit[0].bits_per_word = 8;
	sit[0].delay_usecs = 0;
	sit[0].cs_change = 0;

	rvv = ioctl(fd, SPI_IOC_MESSAGE(1), sit);

	free(items); 
	return(rvv);
}


/*        Read: RW=0 7-bit address, read 8 bit data */
/* Burst  Read: RW=0 7-bit address, read 8 bit*N data */

int readN(int fd, uint8_t address, 
	uint8_t *bytes, size_t bmax, size_t *baread) 
{
	int rvv;
	struct spi_ioc_transfer sit[2];
	uint8_t a0;

	a0 = (address & 0x7F);  /* bit 7 clear for read */

	/* write then read in two messages, one for each. */
	sit[0].tx_buf = (uintptr_t) &(a0);
	sit[0].rx_buf = (__u64) 0;
	sit[0].len = 1;
	sit[0].speed_hz = 100000;
	sit[0].bits_per_word = 8;
	sit[0].delay_usecs = 0;
	sit[0].cs_change = 0;

	sit[1].tx_buf = (__u64) 0;
	sit[1].rx_buf = (uintptr_t) bytes;
	sit[1].len = bmax;
	sit[1].speed_hz = 100000;
	sit[1].bits_per_word = 8;
	sit[1].delay_usecs = 0;
	sit[1].cs_change = 0;

	rvv = ioctl(fd, SPI_IOC_MESSAGE(2), sit);

	return(rvv);
}



/* Set the SPI device as appropriate */
int setupspidev(int fd, uint8_t mode, char *info, size_t infomax)
{
	// uint8_t mode;
	uint8_t bits;
	uint32_t speed;
	char errbuf[128];
	int oks;

	/* The mode we will use here is mode 2: and MSB first which is default. */
	// mode = SPI_MODE_2; 
	/* Max31855 wants mode 1 */
	// mode = SPI_MODE_1; 

	oks = ioctl(fd, SPI_IOC_WR_MODE, &mode);
	if(oks < 0)
	{
		strerror_r(errno, errbuf, 128);
		snprintf(info, infomax, "Setting device mode to 0x%x: %s", mode, errbuf); 
		return(-1);
	}

	bits = 8;
	oks = ioctl(fd, SPI_IOC_WR_BITS_PER_WORD, &bits);
	if(oks < 0)
	{
		strerror_r(errno, errbuf, 128);
		snprintf(info, infomax, "Setting %d bits per word: %s", bits, errbuf); 
		return(-1);
	}

	speed = 100000;
	oks = ioctl(fd, SPI_IOC_WR_MAX_SPEED_HZ, &speed);
	if(oks < 0)
	{
		strerror_r(errno, errbuf, 128);
		snprintf(info, infomax, "Setting speed to %u Hz: %s", speed, errbuf); 
		return(-1);
	}

	snprintf(info, infomax, "OK."); 
	return(0); 
}


#if(0)

/* Example for ad9837 */

int ad9837setfreq(int fd, double fout, int fchan, double *pcfo)
{
	double P228, dfreg; 
	uint32_t ifreg;
	uint16_t Hf, Lf, Cf;
	int oks;

	P228 = pow(2.0, 28.0);  /* 2.68E8 */ 

	/* Set a frequency */
	dfreg = fout * P228 / FMCLK;			/* Accumulator-value, increases with increasing frequency */
	ifreg = (uint32_t) floor(dfreg + 0.5); 

	// printf("%s(%d): F desired = %G set register %G %u 0x%x\n", __FILE__, __LINE__, fout, dfreg, ifreg, ifreg);

	// 0   0  1   0   FSEL PSEL 0   0     0      0       OPBITEN 0   DIV2 0   MODE  0    ; B28 means set 28 bits in two goes
	// 0   1  F13 F12 F11  F10  F09 F08   F07    F06     F05     F04 F03  F02 F01   F00  ; Register 0 Least significant. 
	// 0   1  F27 F26 F25  F24  F23 F22   F21    F20     F19     F18 F17  F16 F15   F14  ; Register 0 Most significant.

	Cf = 0x2000 | ( (fchan & 1) << 11); 
	Hf = ((ifreg >> 14) & 0x3fff) | 0x4000; 
	Lf = (ifreg & 0x3fff) | 0x4000; 

	oks = write16(fd, Cf);
	// printf("Write 0x%x = %d e=%d\n", 0x2000, oks, errno);
	oks = write16(fd, Lf);
	// printf("Write 0x%x = %d e=%d\n", Lf, oks, errno);
	oks = write16(fd, Hf);
	// printf("Write 0x%x = %d e=%d\n", Hf, oks, errno);

	*pcfo = FMCLK * (double) ifreg / P228; 

	return(oks); 
}

#endif

/* Read 32 bits in one go from the device */
int read32(int fd, uint32_t *bits32)
{
	int rvv;
	struct spi_ioc_transfer sit[1];
	uint8_t items[5];

	memset(&sit, 0, sizeof(sit));

	sit[0].tx_buf = (__u64) 0;
	sit[0].rx_buf = (uintptr_t) items;
	sit[0].len = 4;
	sit[0].speed_hz = 100000;
	sit[0].bits_per_word = 8;
	sit[0].delay_usecs = 0;
	sit[0].cs_change = 0;

	rvv = ioctl(fd, SPI_IOC_MESSAGE(1), sit);
	// fprintf(stderr, "%s(%d): Read file %d = %d\n", __FILE__, __LINE__, fd, rvv ); 

	(*bits32) = (items[3]) | (items[2] << 8) | (items[1] << 16) | (items[0] << 24);

	return(rvv);
}








